@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2025 Patrizio Bekerle -- <patrizio@bekerle.com>
+ * Copyright (c) 2014-2026 Patrizio Bekerle -- <patrizio@bekerle.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -119,6 +119,11 @@ void copyCodeBlockText(const QTextBlock &initialBlock);
 
 bool toggleCheckBoxAtCursor(QPlainTextEdit *textEdit);
 bool autoFormatTableAtCursor(QPlainTextEdit *textEdit);
+bool isTableAtCursor(QPlainTextEdit *textEdit, int *cursorColumn = nullptr);
+bool insertTableColumnLeft(QPlainTextEdit *textEdit);
+bool insertTableColumnRight(QPlainTextEdit *textEdit);
+bool insertTableRowAbove(QPlainTextEdit *textEdit);
+bool insertTableRowBelow(QPlainTextEdit *textEdit);
 
 void updateInterfaceFontSize(int fontSize = -1);
 
@@ -141,6 +146,7 @@ bool doLinuxDarkModeCheck();
 
 QIcon folderIcon();
 QIcon noteIcon();
+QIcon favoriteNoteIcon();
 QIcon tagIcon();
 
 /**
